.colorBarContainer[data-v-752aacf2]{position:relative;height:20px;max-width:400px;min-width:255px;flex:1;border-radius:2px;overflow:hidden;margin:0 5px}.colorBar[data-v-752aacf2]{position:absolute;top:0;left:0;height:100%;width:100%}.textField[data-v-752aacf2]{max-width:80px}